/**
 * @file	main.c
 * @brief	Play CS47L63 test tone
 * 
 * @details	Declares an array containing register values that configure the
 * 		codec, a single `config_regs()` function that iterates and
 * 		writes that array, then a simple `main()` entry point that
 * 		initializes the codec and enables the tone generator for 2 secs.
 * 
 * @note	SEE LINE 61 FOR PROBLEM: ONLY WORKS WITH DEBUG OPTIMIZATIONS!
 */

#include <zephyr/kernel.h>
#include "cs47l63_comm.h"


static cs47l63_t cs47l63_driver;


static const uint32_t cs47l63_configuration[][2] = {

	/* Configure clocks to use internal oscillator for FLL1 */
	{ CS47L63_RCO_CTRL1,         0x00000001 },  /* EN=1 */
	{ CS47L63_FLL1_CONTROL3,     0x00010000 },  /* LAMBDA=1 */
	{ CS47L63_FLL1_CONTROL2,     0x88202004 },  /* REFCLK_SRC=RCO */
	{ CS47L63_FLL1_CONTROL1,     0x00000007 },  /* EN=1, CTRL_UPD=1 */
	{ CS47L63_SYSTEM_CLOCK1,     0x00000444 },  /* EN=1 */

	/* Configure output to play tone generator */
	{ CS47L63_OUT1L_INPUT1,      0x00800004 },  /* SRC=TONE1 */
	{ CS47L63_OUT1L_VOLUME_1,    0x00000280 },  /* MUTE=0, VU=1 */
	{ CS47L63_OUTPUT_ENABLE_1,   0x00000002 },  /* EN=1 */
};


/**
 * @brief	Write an array to multiple CS47L63 registers
 * @param[in]	config: Array containing address/value pairs
 * @param[in]	count:  Number of registers to write
 */
static void config_regs(const uint32_t config[][2], uint32_t count)
{
	uint32_t reg;
	uint32_t val;

	for (int i = 0; i < count; i++) {
		reg = config[i][0];
		val = config[i][1];
		cs47l63_write_reg(&cs47l63_driver, reg, val);
	}
}



int main(void)
{
	int ret;

	/* Initialize hardware */
	/* ONLY RETURNS 0 IF DEBUG OPTIMIZATIONS ARE ENABLED IN BUILD CONFIG! */
	ret = cs47l63_comm_init(&cs47l63_driver);
	printk("cs47l63_comm_init() = %d\n", ret);


	/* Configure registers */
	config_regs(cs47l63_configuration, ARRAY_SIZE(cs47l63_configuration));

	/* Enable tone generator for 2 seconds */
	cs47l63_write_reg(&cs47l63_driver, CS47L63_TONE_GENERATOR1, 0x01);
	k_sleep(K_MSEC(2000));

	/* Disable tone generator */
	cs47l63_write_reg(&cs47l63_driver, CS47L63_TONE_GENERATOR1, 0x00);

}
